"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.Component = exports.merge = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _mithril = require("mithril");

var _mithril2 = _interopRequireDefault(_mithril);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var LIFECYCLE_METHODS = ["oninit", "oncreate", "onbeforeupdate", "onupdate", "onbeforeremove", "onremove"];

var assign = Object.assign;

var isObject = function isObject(data) {
	return data != null && (typeof data === "undefined" ? "undefined" : _typeof(data)) === 'object' && isArray(data) === false;
};

var isArray = Array.isArray;

var pickBy = function pickBy(obj, checker) {
	return Object.keys(obj).reduce(function (desiredObj, key) {
		var value = obj[key];

		if (checker(key)) {
			desiredObj[key] = value;
		}

		return desiredObj;
	}, {});
};

var merge = exports.merge = function merge(destination, source) {
	Object.keys(source).forEach(function (key) {
		if (isObject(source[key])) {
			destination[key] = merge(isObject(destination[key]) && destination[key] || {}, source[key]);
		} else {
			destination[key] = source[key];
		}
	});

	return destination;
};

var Component = exports.Component = function () {
	_createClass(Component, [{
		key: "getComponentName",
		value: function getComponentName() {
			return this.constructor.displayName || this.constructor.name;
		}
	}]);

	function Component() {
		_classCallCheck(this, Component);

		this.mixins = this.mixins || [];
		this.mixins.reduce(function (acc, mixin) {
			assign(acc, mixin);
			return acc;
		}, this);
	}
	/*
  * Generates stylesheet based upon data returned by getStyle()
  * */


	_createClass(Component, [{
		key: "genStyle",
		value: function genStyle(jsStyle) {
			var _this = this;

			var componentName = this.getComponentName();
			var genSingleLevel = function genSingleLevel(js) {
				var indent = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;

				var leftPad = new Array(indent).join(" ");
				var css = "";

				for (var key in js) {
					if (js.hasOwnProperty(key)) {
						if (_typeof(js[key]) === "object") {
							css += leftPad + _this.localizeSelector(key, componentName) + " {\n";

							css += genSingleLevel(js[key], indent + 2);

							css += leftPad + "}\n";
						} else {
							// convert camelcase to snake case
							var normalizedKey = key.replace(/([A-Z])/g, "-$1").toLowerCase();
							css += leftPad + normalizedKey + ": " + js[key] + ";\n";
						}
					}
				}

				return css;
			};

			return "\n" + genSingleLevel(jsStyle);
		}

		/*
   * Attaches component name to the key.
   * This increases specificity.
   * */

	}, {
		key: "localizeSelector",
		value: function localizeSelector(key, componentName) {
			var _this2 = this;

			if (key.indexOf(":") !== -1) {
				var frags = key.split(":");
				frags[0] = this.localizeSelector(frags[0], componentName);
				return frags.join(":");
			}

			if (key.indexOf(",") !== -1) {
				return key.split(",").map(function (frag) {
					return _this2.localizeSelector(frag.trim(), componentName);
				}).join(", ");
			}

			return key.replace(/^([^@,%\s]*?)$/, "$1[data-component=" + componentName + "]").replace(/^([^@]*?)\s+(.*?)$/, "$1[data-component=" + componentName + "] $2")
			// reverse for keyframe keys
			.replace(/^(from\[.*?)$/, "from").replace(/^(to\[.*?)$/, "to");
		}

		/*
   * Returns json which will be used by genStyles() to generate stylesheet for this component.
   * */

	}, {
		key: "getStyle",
		value: function getStyle(vnode) {}

		/*
   * Attach styles to the head
   * */

	}, {
		key: "attachStyle",
		value: function attachStyle(style) {
			var componentName = this.getComponentName();
			var node = document.createElement("style");
			node.id = componentName + "-style";

			if (node.styleSheet) {
				node.styleSheet.cssText = style;
			} else {
				node.appendChild(document.createTextNode(style));
			}

			document.getElementsByTagName('head')[0].appendChild(node);
		}

		/*
   * Returns true for attirbutes which are selected for root dom of the component.
   * */

	}, {
		key: "isRootAttr",
		value: function isRootAttr(key) {
			if (LIFECYCLE_METHODS.indexOf(key) !== -1) return false;

			try {
				return (/^(key|id|style|on.*|data-.*|config)$/.test(key) ? true : false
				);
			} catch (err) {
				if (err instanceof TypeError) {
					return false;
				}
			}
		}
	}, {
		key: "insertUserClass",
		value: function insertUserClass(classList, userClass) {
			if (classList.length == 0) {
				return [userClass];
			} else if (classList.length == 1) {
				classList.unshift(userClass);
				return classList;
			} else {
				classList.splice(1, 0, userClass);
				return classList;
			}
		}
	}, {
		key: "getClass",
		value: function getClass(classList, userClass) {
			return this.insertUserClass(classList, userClass).filter(function (aClass) {
				return [null, undefined, "", false].indexOf(aClass) === -1;
			}).join(" ");
		}
	}, {
		key: "getAttrs",
		value: function getAttrs(vnode) {
			var defaultAttrs = this.getDefaultAttrs(vnode) || {};
			var newAttrs = assign(defaultAttrs, vnode.attrs);

			newAttrs.rootAttrs = [defaultAttrs.rootAttrs || {}, newAttrs.rootAttrs || {}, { "data-component": this.getComponentName() }, pickBy(newAttrs, this.isRootAttr)].reduce(merge, {});

			var newClassName = this.getClass(this.getClassList({ attrs: newAttrs, children: vnode.children, state: vnode.state }), newAttrs.class);
			if (newClassName) {
				newAttrs.rootAttrs.class = newClassName;
			}

			return newAttrs;
		}
	}, {
		key: "getDefaultAttrs",
		value: function getDefaultAttrs(vnode) {
			return {};
		}
	}, {
		key: "getClassList",
		value: function getClassList(vnode) {
			return [];
		}
	}, {
		key: "validateAttrs",
		value: function validateAttrs(attrs) {}
	}, {
		key: "cacheVnode",
		value: function cacheVnode(vnode) {
			this.vnode = vnode;
		}
	}, {
		key: "redraw",
		value: function redraw() {
			_mithril2.default.render(this.vnode.dom, this.isolatedView(this.vnode));
		}
	}, {
		key: "oninit",
		value: function oninit(vnode) {
			vnode.attrs = vnode.attrs || {};
			vnode.attrs = this.getAttrs(vnode);
			this.validateAttrs(vnode.attrs);

			this.cacheVnode(vnode);

			var style = this.getStyle(vnode);
			var cName = this.getComponentName();

			if (!style || style && document.getElementById(cName + "-style")) return;

			this.attachStyle(this.genStyle(style, cName), cName);
		}
	}, {
		key: "oncreate",
		value: function oncreate(vnode) {
			this.isolatedView && this.redraw();
		}
	}, {
		key: "onupdate",
		value: function onupdate(vnode) {
			this.isolatedView && this.redraw();
		}
	}, {
		key: "onbeforeupdate",
		value: function onbeforeupdate(vnode) {
			vnode.attrs = vnode.attrs || {};
			vnode.attrs = this.getAttrs(vnode);
			this.validateAttrs(vnode.attrs);
			this.cacheVnode(vnode);
		}
	}]);

	return Component;
}();

;